package com.yovinchen.xlcs.home.service.impl;

import com.yovinchen.xlcs.client.product.ProductFeignClient;
import com.yovinchen.xlcs.client.search.SkuFeignClient;
import com.yovinchen.xlcs.client.user.UserFeignClient;
import com.yovinchen.xlcs.home.service.HomeService;
import com.yovinchen.xlcs.model.product.Category;
import com.yovinchen.xlcs.model.product.SkuInfo;
import com.yovinchen.xlcs.model.search.SkuEs;
import com.yovinchen.xlcs.vo.user.LeaderAddressVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.concurrent.*;

/**
 * ClassName: HomeServiceImpl
 * Package: com.yovinchen.xlcs.home.service.impl
 *
 * @author yovinchen
 * @since 2023/9/25 12:12
 */
@Service
@Slf4j
public class HomeServiceImpl implements HomeService {


    @Autowired
    private UserFeignClient userFeignClient;

    @Autowired
    private ProductFeignClient productFeignClient;

    @Autowired
    private SkuFeignClient skuFeignClient;

    /**
     * 获取首页数据
     *
     * @param userId
     * @return
     */
    @Override
    public Map<String, Object> homeData(Long userId) {
        Map<String, Object> result = new ConcurrentHashMap<>();

        ExecutorService executorService = Executors.newFixedThreadPool(4);
        ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(4);

        // 1 根据userId获取当前登录用户地址信息
        CompletableFuture<Void> getAddressFuture = CompletableFuture.runAsync(() -> {
            try {
                LeaderAddressVo leaderAddressVo = userFeignClient.getUserAddressByUserId(userId);
                if (leaderAddressVo != null) {
                    result.put("leaderAddressVo", leaderAddressVo);
                }
            } catch (Exception e) {
                log.error("获取地址信息任务出错: {}", e.getMessage(), e);
            }
        }, executorService);

        // 设置获取地址信息任务的超时
        scheduler.schedule(() -> {
            if (!getAddressFuture.isDone()) {
                getAddressFuture.complete(null);
                log.info("获取地址信息任务超时");
            }
        }, 1, TimeUnit.SECONDS);

        // 2 获取所有分类
        CompletableFuture<Void> getCategoryFuture = CompletableFuture.runAsync(() -> {
            try {
                List<Category> categoryList = productFeignClient.findAllCategoryList();
                if (categoryList != null) {
                    result.put("categoryList", categoryList);
                }
            } catch (Exception e) {
                log.error("获取所有分类任务出错: {}", e.getMessage(), e);
            }
        }, executorService);

        // 设置获取所有分类任务的超时
        scheduler.schedule(() -> {
            if (!getCategoryFuture.isDone()) {
                getCategoryFuture.complete(null);
                log.info("获取所有分类任务超时");
            }
        }, 1, TimeUnit.SECONDS);

        // 3 获取新人专享商品
        CompletableFuture<Void> getNewPersonSkuInfoFuture = CompletableFuture.runAsync(() -> {
            try {
                List<SkuInfo> newPersonSkuInfoList = productFeignClient.findNewPersonSkuInfoList();
                if (newPersonSkuInfoList != null) {
                    result.put("newPersonSkuInfoList", newPersonSkuInfoList);
                }
            } catch (Exception e) {
                log.error("获取新人专享商品任务出错: {}", e.getMessage(), e);
            }
        }, executorService);

        // 设置获取新人专享商品任务的超时
        scheduler.schedule(() -> {
            if (!getNewPersonSkuInfoFuture.isDone()) {
                getNewPersonSkuInfoFuture.complete(null);
                log.info("获取新人专享商品任务超时");
            }
        }, 1, TimeUnit.SECONDS);

        // 4 获取爆款商品
        CompletableFuture<Void> getHotSkuFuture = CompletableFuture.runAsync(() -> {
            try {
                List<SkuEs> hotSkuList = skuFeignClient.findHotSkuList();
                if (hotSkuList != null) {
                    result.put("hotSkuList", hotSkuList);
                }
            } catch (Exception e) {
                log.error("获取爆款商品任务出错: {}", e.getMessage(), e);
            }
        }, executorService);

        // 设置获取爆款商品任务的超时
        scheduler.schedule(() -> {
            if (!getHotSkuFuture.isDone()) {
                getHotSkuFuture.complete(null);
                log.info("获取爆款商品任务超时");
            }
        }, 1, TimeUnit.SECONDS);

        // 等待所有任务完成
        CompletableFuture<Void> allFutures = CompletableFuture.allOf(getAddressFuture, getCategoryFuture, getNewPersonSkuInfoFuture, getHotSkuFuture);

        try {
            allFutures.get(); // 等待所有任务完成
        } catch (InterruptedException | ExecutionException e) {
            log.error("任务执行出错: {}", e.getMessage(), e);
        } finally {
            executorService.shutdown(); // 关闭线程池
            scheduler.shutdown(); // 关闭调度器
        }

        return result;
    }


}
