package com.yovinchen.xlcs.payment.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.AlipayConstants;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipayTradePagePayRequest;
import com.alipay.api.response.AlipayTradePagePayResponse;
import com.yovinchen.xlcs.client.order.OrderFeignClient;
import com.yovinchen.xlcs.model.order.OrderInfo;
import com.yovinchen.xlcs.model.order.PaymentInfo;
import com.yovinchen.xlcs.payment.service.AlipayService;
import com.yovinchen.xlcs.payment.service.PaymentInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * ClassName: AlipayServiceImpl
 * Package: com.yovinchen.xlcs.payment.service.impl
 *
 * @author yovinchen
 * @since 2024/2/22 17:04
 */
@Service
@Slf4j
public class AlipayServiceImpl implements AlipayService {

    @Autowired
    private PaymentInfoService paymentInfoService;

    @Autowired
    private RedisTemplate redisTemplate;

    @Resource
    private AlipayClient alipayClient;
    @Autowired
    private OrderFeignClient orderFeignClient;
    @Resource
    private Environment config;

    @Transactional
    @Override
    public Map<String, String> createPay(String orderNo) {
        try {
            //1 向payment_info支付记录表添加记录，目前支付状态：正在支付中
            PaymentInfo paymentInfo = paymentInfoService.getPaymentInfoByOrderNo(orderNo);
            if (paymentInfo == null) {
                paymentInfo = paymentInfoService.savePaymentInfo(orderNo);
            }

            //2 调用支付宝支付
            AlipayTradePagePayRequest request = new AlipayTradePagePayRequest();
//            AlipayTradePagePayModel bizModel=new AlipayTradePagePayModel();
//            //异步接收地址，仅支持http/https，公网可访问
//            request.setNotifyUrl("");
//            //同步跳转地址，仅支持http/https
//            request.setReturnUrl("");

            JSONObject bizContent = new JSONObject();
            //商户订单号，商家自定义，保持唯一性
            bizContent.put("out_trade_no", paymentInfo.getOrderNo());
//            bizModel.setOutTradeNo(paymentInfo.getOrderNo());
            //支付金额，最小值0.01元
            bizContent.put("total_amount", paymentInfo.getTotalAmount());
//            bizModel.setTotalAmount(String.valueOf(paymentInfo.getTotalAmount()));
            //订单标题，不可使用特殊符号
            bizContent.put("subject", paymentInfo.getSubject());
//            bizModel.setOutTradeNo(paymentInfo.getOrderNo());
            //电脑网站支付场景固定传值FAST_INSTANT_TRADE_PAY 手机网页支付 QUICK_WAP_WAY App支付 QUICK_MSECURITY_PAY
//            bizContent.put("product_code", "FAST_INSTANT_TRADE_PAY");
            bizContent.put("product_code", "QUICK_WAP_WAY");
//            request.setBizModel(bizModel);

            // TODO 跳转到成功页面
//            request.setNotifyUrl(config.getClass());

            request.setBizContent(bizContent.toString());
            AlipayTradePagePayResponse response = alipayClient.pageExecute(request, "POST");
            // 如果需要返回GET请求，请使用
//             AlipayTradePagePayResponse response = alipayClient.pageExecute(request,"GET");
            String pageRedirectionData = response.getBody();
            System.out.println(pageRedirectionData);

            //执行请求，调用支付宝接口

            if (response.isSuccess()) {
                log.info("调用成功，返回结果 ===> " + response.getBody());
                Map<String, String> map = new HashMap();
                map.put("formStr", response.getBody());
                return map;
            } else {
                log.info("调用失败，返回码 ===> " + response.getCode() + ", 返回描述 ===> " + response.getMsg());
                throw new RuntimeException("创建支付交易失败");
            }
        } catch (AlipayApiException e) {
            e.printStackTrace();
            throw new RuntimeException("创建支付宝支付交易失败");
        }
    }

    @Override
    public String tradeNotify(Map<String, String> params) {

        log.info("支付通知正在执行");
        log.info("通知参数 ===> {}", params);

        String result = "failure";

        try {
            //异步通知验签
            boolean signVerified = AlipaySignature.rsaCheckV1(params, config.getProperty("alipay.alipay-public-key"), AlipayConstants.CHARSET_UTF8, AlipayConstants.SIGN_TYPE_RSA2); //调用SDK验证签名

            if (!signVerified) {
                //验签失败则记录异常日志，并在response中返回failure.
                log.error("支付成功异步通知验签失败！");
                return result;
            }

            // 验签成功后
            log.info("支付成功异步通知验签成功！");

            //按照支付结果异步通知中的描述，对支付结果中的业务内容进行二次校验，
            //1 商户需要验证该通知数据中的 out_trade_no 是否为商户系统中创建的订单号
            String outTradeNo = params.get("out_trade_no");
            OrderInfo order = orderFeignClient.getOrderInfo(outTradeNo);
            if (order == null) {
                log.error("订单不存在");
                return result;
            }

            //2 判断 total_amount 是否确实为该订单的实际金额（即商户订单创建时的金额）
            String totalAmount = params.get("total_amount");
            int totalAmountInt = new BigDecimal(totalAmount).multiply(new BigDecimal("100"))
                    .intValue();
            int totalFeeInt = order.getTotalAmount()
                    .intValue();
            if (totalAmountInt != totalFeeInt) {
                log.error("金额校验失败");
                return result;
            }

            //3 校验通知中的 seller_id（或者 seller_email) 是否为 out_trade_no 这笔单据的对应的操作方
            String sellerId = params.get("seller_id");
            String sellerIdProperty = config.getProperty("alipay.seller-id");
            if (!sellerId.equals(sellerIdProperty)) {
                log.error("商家pid校验失败");
                return result;
            }

            //4 验证 app_id 是否为该商户本身
            String appId = params.get("app_id");
            String appIdProperty = config.getProperty("alipay.app-id");
            if (!appId.equals(appIdProperty)) {
                log.error("appid校验失败");
                return result;
            }

            //在支付宝的业务通知中，只有交易通知状态为 TRADE_SUCCESS时，
            // 支付宝才会认定为买家付款成功。
            String tradeStatus = params.get("trade_status");
            if (!"TRADE_SUCCESS".equals(tradeStatus)) {
                log.error("支付未成功");
                return result;
            }

            //处理业务 修改订单状态 记录支付日志
//            aliPayService.processOrder(params);
            paymentInfoService.paySuccess(outTradeNo, params);
            //校验成功后在response中返回success并继续商户自身业务处理，校验失败返回failure
            result = "success";
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        return result;
    }
}
