package com.yovinchen.xlcs.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yovinchen.xlcs.common.auth.AuthContextHolder;
import com.yovinchen.xlcs.common.exception.xlcsException;
import com.yovinchen.xlcs.common.result.ResultCodeEnum;
import com.yovinchen.xlcs.common.utils.MD5;
import com.yovinchen.xlcs.enums.UserType;
import com.yovinchen.xlcs.model.user.Leader;
import com.yovinchen.xlcs.model.user.User;
import com.yovinchen.xlcs.model.user.UserDelivery;
import com.yovinchen.xlcs.user.mapper.LeaderMapper;
import com.yovinchen.xlcs.user.mapper.UserDeliveryMapper;
import com.yovinchen.xlcs.user.mapper.UserMapper;
import com.yovinchen.xlcs.user.service.UserService;
import com.yovinchen.xlcs.vo.user.LeaderAddressVo;
import com.yovinchen.xlcs.vo.user.UserLoginVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * ClassName: UserServiceImpl
 * Package: com.yovinchen.xlcs.user.service.impl
 *
 * @author yovinchen
 * @since 2023/9/22 16:34
 */
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {
    @Autowired
    private UserDeliveryMapper userDeliveryMapper;

    @Autowired
    private LeaderMapper leaderMapper;

    /**
     * 使用正则表达式验证手机号格式
     *
     * @param phoneNumber
     * @return
     */
    public boolean isValidPhoneNumber(String phoneNumber) {
        // 使用正则表达式验证手机号格式
        String regex = "^1[3456789]\\d{9}$"; // 手机号的正则表达式
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(phoneNumber);
        return matcher.matches();
    }

    /**
     * 判断是否是第一次使用微信授权登录：如何判断？openId
     *
     * @param openid
     * @return
     */
    @Override
    public User getUserByOpenId(String openid) {
        User user = baseMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getOpenId, openid));
        return user;
    }

    /**
     * 根据userId查询提货点和配送员信息
     *
     * @param userId
     * @return
     */
    @Override
    public LeaderAddressVo getLeaderAddressByUserId(Long userId) {
        //根据userId查询用户默认的配送员id
        UserDelivery userDelivery = userDeliveryMapper.selectOne(new LambdaQueryWrapper<UserDelivery>().eq(UserDelivery::getUserId, userId)
                .eq(UserDelivery::getIsDefault, 1));
        if (userDelivery == null) {
            return null;
        }
        //拿着上面查询配送员id查询leader表查询配送员其他信息
        Leader leader = leaderMapper.selectById(userDelivery.getLeaderId());
        //封装数据到LeaderAddressVo
        LeaderAddressVo leaderAddressVo = new LeaderAddressVo();
        BeanUtils.copyProperties(leader, leaderAddressVo);
        leaderAddressVo.setUserId(userId);
        leaderAddressVo.setLeaderId(leader.getId());
        leaderAddressVo.setLeaderName(leader.getName());
        leaderAddressVo.setLeaderPhone(leader.getPhone());
        leaderAddressVo.setWareId(userDelivery.getWareId());
        leaderAddressVo.setStorePath(leader.getStorePath());
        return leaderAddressVo;
    }

    /**
     * 获取当前登录用户信息
     *
     * @param id
     * @return
     */
    @Override
    public UserLoginVo getUserLoginVo(Long id) {
        User user = baseMapper.selectById(id);
        UserLoginVo userLoginVo = new UserLoginVo();
        userLoginVo.setUserId(id);
        userLoginVo.setNickName(user.getNickName());
        userLoginVo.setPhotoUrl(user.getPhotoUrl());
        userLoginVo.setIsNew(user.getIsNew());
        userLoginVo.setOpenId(user.getOpenId());

        UserDelivery userDelivery = userDeliveryMapper.selectOne(new LambdaQueryWrapper<UserDelivery>().eq(UserDelivery::getUserId, id)
                .eq(UserDelivery::getIsDefault, 1));
        if (userDelivery != null) {
            userLoginVo.setLeaderId(userDelivery.getLeaderId());
            userLoginVo.setWareId(userDelivery.getWareId());
        } else {
            userLoginVo.setLeaderId(1L);
            userLoginVo.setWareId(1L);
        }
        return userLoginVo;
    }

    /**
     * 更新用户信息
     *
     * @param user
     */
    @Override
    public void updateUser(User user) {
        //获取当前登录用户id
        User user1 = baseMapper.selectById(AuthContextHolder.getUserId());
        //把昵称更新为微信用户
        user1.setNickName(user.getNickName()
                .replaceAll("[ue000-uefff]", "*"));
        user1.setPhotoUrl(user.getPhotoUrl());
        baseMapper.updateById(user1);
    }

    /**
     * 手机号注册账号
     *
     * @param phone
     * @param password
     * @return
     */
    @Override
    public Map<String, Object> register(String phone, String password) {

        // 检测手机号格式
        if (!isValidPhoneNumber(phone)) {
            throw new xlcsException(ResultCodeEnum.PHONE_FORMAT_ERROR);
        }

        User user = baseMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getPhone, phone));

        // 查看用户信息是否存在
        if (user != null) {
            throw new xlcsException(ResultCodeEnum.PHONE_REGISTERED_ERROR);
        }

        //TODO 短信验证


        //构建用户信息
        user = new User();
        user.setNickName(phone);
        user.setPhone(phone);
        user.setPhotoUrl("");
        user.setPassword(MD5.encrypt(password));
        user.setUserType(UserType.USER);
        user.setIsNew(1);
        baseMapper.insert(user);
        return null;
    }

}
